#define G_LOG_DOMAIN "gpod-feed-view"

#include "config.h"

#include "gpod-feed.h"
#include "gpod-feed-view.h"

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-feed-view
 * @title: GpodFeedView
 * @short_description: A #GtkBox widget for list feeds
 * @include: gpod-feed-view.h
 * @see_also: #GpodFeed, #GpodFeedWidget
 *
 * The #GpodFeedView is an instance #GtkBox, and acts as the primary
 * window/widget for listing feeds in the #GpodWindow<!-- -->'s #GtkStack.
 **/

typedef struct _GpodFeedViewPrivate
{
    GtkWidget *flow_box;
} GpodFeedViewPrivate;

struct _GpodFeedView
{
    GtkBox parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodFeedView, gpod_feed_view, GTK_TYPE_BOX)


static void
add (GpodFeedView *self,
     GpodFeed     *feed)
{
    GpodFeedViewPrivate *priv = gpod_feed_view_get_instance_private (self);
    GtkBuilder *builder = gtk_builder_new ();
    GError *error = NULL;
    GtkWidget *item;
    GtkLabel *title;
    GtkImage *image;

    g_assert (GPOD_IS_FEED_VIEW (self));
    g_assert (GPOD_IS_FEED (feed));
    g_assert (GTK_IS_FLOW_BOX (priv->flow_box));

    if (!gtk_builder_add_from_resource (builder, PACKAGE_RESOURCES "/ui/widgets/feed-view-item.ui", &error))
    {
        g_warning ("Unable to add feed to FeedView: %s", error->message);
        g_error_free (error);
        return;
    }

    item = GTK_WIDGET (g_object_ref (gtk_builder_get_object (builder, "box")));
    title = GTK_LABEL (g_object_ref (gtk_builder_get_object (builder, "title")));
    image = GTK_IMAGE (g_object_ref (gtk_builder_get_object (builder, "image")));

    gtk_label_set_label (title, gpod_feed_get_title (feed));
    //gtk_image_set_from_pixbuf (image, "temp");

    //preview_icon_surface = gdk_cairo_surface_create_from_pixbuf (preview_icon, scale, NULL);
    //gtk_image_set_from_surface (image,

    gtk_flow_box_insert (GTK_FLOW_BOX (priv->flow_box),
                         GTK_WIDGET (item),
                         -1);

    /* Clean-up */
    g_clear_object (&builder);
}

/*
 * gpod_feed_view_finalize:
 * @object: An instance of #GpodFeedView
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_feed_view_finalize (GObject *object)
{
    G_OBJECT_CLASS (gpod_feed_view_parent_class)->finalize(object);
}

/*
 * gpod_feed_view_init:
 * @self: An instance of #GpodFeedView
 *
 * Initializes various aspects of the #GpodFeedView. Anything the #GpodFeedView
 * would need before becoming presented should be done here.
 */
static void
gpod_feed_view_init (GpodFeedView *self)
{
    /* Init views/feeds.ui template */
    gtk_widget_init_template (GTK_WIDGET (self));
}

/*
 * gpod_feed_view_class_init:
 * @klass: An instance of #GpodFeedViewClass
 *
 * Initializes the various virtural functions in #GpodFeedViewClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_feed_view_class_init (GpodFeedViewClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

    object_class->finalize = gpod_feed_view_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/views/feed.ui");

    gtk_widget_class_bind_template_child_private (widget_class, GpodFeedView, flow_box);
}

/**
 * gpod_feed_view_new:
 *
 * Creates a new #GpodFeedView, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodFeedView *feed_view;
 *
 * feed_view = gpod_feed_view_new ();
 * ]|
 *
 * Returns: #GpodFeedView
 */
GpodFeedView *
gpod_feed_view_new ()
{
    return g_object_new (GPOD_TYPE_FEED_VIEW,
                         NULL);
}

void
gpod_feed_view_add (GpodFeedView *self,
                    GpodFeed     *feed)
{
    add (self, feed);
}
